class NotFound(Exception):
    def __init__(self, text):
        pass


class NotSupported(Exception):
    def __init__(self, text):
        pass


class ParseError(Exception):
    def __init__(self, text):
        pass
